# guidocioni/eumetsat-python
import numpy as np                       # Import the Numpy package
import colorsys


def load_CPT(path):
    try:
        f = open(path)
    except:
        print ("File ", path, "not found")
        return None

    lines = f.readlines()

    f.close()

    x = np.array([])
    r = np.array([])
    g = np.array([])
    b = np.array([])

    colorModel = 'RGB'

    for l in lines:
        ls = l.split()
        if l[0] == '#':
            if ls[-1] == 'HSV':
                colorModel = 'HSV'
                continue
            else:
                continue
        if ls[0] == 'B' or ls[0] == 'F' or ls[0] == 'N':
            pass
        else:
            x=np.append(x,float(ls[0]))
            r=np.append(r,float(ls[1]))
            g=np.append(g,float(ls[2]))
            b=np.append(b,float(ls[3]))
            xtemp = float(ls[4])
            rtemp = float(ls[5])
            gtemp = float(ls[6])
            btemp = float(ls[7])

        x=np.append(x,xtemp)
        r=np.append(r,rtemp)
        g=np.append(g,gtemp)
        b=np.append(b,btemp)

    if colorModel == 'HSV':
        for i in range(r.shape[0]):
            rr, gg, bb = colorsys.hsv_to_rgb(r[i]/360.,g[i],b[i])
        r[i] = rr ; g[i] = gg ; b[i] = bb

    if colorModel == 'RGB':
        r = r/255.0
        g = g/255.0
        b = b/255.0

    xNorm = (x - x[0])/(x[-1] - x[0])

    red   = []
    blue  = []
    green = []

    for i in range(len(x)):
        red.append([xNorm[i],r[i],r[i]])
        green.append([xNorm[i],g[i],g[i]])
        blue.append([xNorm[i],b[i],b[i]])

    colorDict = {'red': red, 'green': green, 'blue': blue}

    return colorDict


import os
import cartopy
import cartopy.crs as ccrs
import matplotlib.pyplot as plt
from cima.goes.utils.load_CPT import load_CPT
try:
    import cupy as cp
    import numpy as np
    asnumpy = np.asnumpy
except:
    import numpy as np
    # import numpy as cp
    asnumpy = lambda x: x


LOCAL_BASE_PATH = os.path.dirname(os.path.abspath(__file__))


def add_cultural(ax):
    states_provinces = cartopy.feature.NaturalEarthFeature(
        category='cultural',
        name='admin_1_states_provinces_lines',
        scale='10m',
        facecolor='none')

    countries = cartopy.feature.NaturalEarthFeature(
        category='cultural',
        name='admin_0_countries',
        scale='10m',
        facecolor='none')

    linewidth = 0.50
    ax.coastlines(resolution='10m', color='white', linewidth=linewidth)
    ax.add_feature(countries, edgecolor='white', linewidth=linewidth)
    ax.add_feature(states_provinces, edgecolor='white', linewidth=linewidth)


def add_grid(ax, lonlat_region):
    linewidth = 1.25
    gl = ax.gridlines(linewidth=linewidth,
                      linestyle='dotted',
                      color='r',
                      crs=ccrs.PlateCarree(),
                      draw_labels=True)


def get_SMN_palette():
    from matplotlib.colors import LinearSegmentedColormap
    filepath = os.path.join(LOCAL_BASE_PATH, 'smn_topes.cpt')
    cpt = load_CPT(filepath)
    return LinearSegmentedColormap('cpt', cpt)


def trim_excess(lonlat_region):
    latsouth = lonlat_region[2]
    latnorth = lonlat_region[3]
    lonwest = lonlat_region[0]
    loneast = lonlat_region[1]
    return [lonwest+0.5, loneast-0.5, latsouth+0.5, latnorth-0.5]


def save_image(image, filepath, tile, lats, lons, cmap='gray', vmin=0, vmax=0.7, cultural=False):
    dirpath = os.path.dirname(os.path.abspath(filepath))
    if not os.path.exists(dirpath):
        os.makedirs(dirpath, exist_ok=True)
    dummy_dpi = 100
    x, y = image.shape
    fig = plt.figure(frameon=False)
    try:
        fig.set_size_inches(x / dummy_dpi, y / dummy_dpi)

        ax = fig.add_subplot(1, 1, 1, projection=ccrs.PlateCarree())
        ax.set_axis_off()
        ax.set_extent(trim_excess(tile.extent), crs=ccrs.PlateCarree())
        if cultural:
            add_cultural(ax)
            add_grid(ax, tile.extent)
        img = ax.pcolormesh(lons, lats, image, cmap=cmap, vmin=vmin, vmax=vmax)

        fig.add_axes(ax, projection=ccrs.PlateCarree())
        ax.axis('off')
        plt.savefig(filepath + '.png', dpi=dummy_dpi, bbox_inches='tight', pad_inches=0)
    finally:
        fig.close()
        plt.close()



#!/usr/bin/env python

from setuptools import find_namespace_packages
from distutils.core import setup

setup(
    name='cima.goes',
    version='1.1.beta22',
    description='GOES-16 File Processing',
    author='Fido Garcia',
    author_email='garciafido@gmail.com',
    package_dir={'': 'src'},
    url='https://github.com/garciafido/cima-goes',
    packages=find_namespace_packages(where='src'),
    include_package_data=True,
    license='MIT',
    package_data={'': ['*.json', '*.cpt']},
    data_files = [("", ["LICENSE"])],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


import os
import asyncio
import time
import psutil
import gc
import concurrent.futures
from cima.goes.gcs import get_datasets, band
from cima.goes.gcs import close_datasets
from cima.goes.tiles import load_tiles, get_data, get_lats_lons, vis_correction
from cima.goes.img import save_image, get_SMN_palette


def _save_one_hour_OTs_images(images_path, year, day_of_year, hour):
    tiles = load_tiles()
    datasets = get_datasets(year, day_of_year, hour, [band.RED, band.CLEAN_LONGWAVE_WINDOW])
    for ds in datasets:
        for tile_number in map(str, range(len(tiles.RED.keys()))):
            filepath_prefix = os.path.join(images_path, f'{ds.start[:4]}/{ds.start[4:7]}/{ds.start}')
            print(filepath_prefix, tile_number)
            print(get_usage())

            lats, lons = get_lats_lons(ds.RED, tiles.RED[tile_number])
            data = get_data(ds.RED, tiles.RED[tile_number])
            gray = vis_correction(data)
            save_image(gray,
                       f'{filepath_prefix}_{tile_number}_vis',
                       tiles.RED[tile_number],
                       lats, lons)

            lats, lons = get_lats_lons(ds.CLEAN_LONGWAVE_WINDOW, tiles.CLEAN_LONGWAVE_WINDOW[tile_number])
            data = get_data(ds.CLEAN_LONGWAVE_WINDOW, tiles.CLEAN_LONGWAVE_WINDOW[tile_number])
            ir = data - 273
            save_image(ir,
                       f'{filepath_prefix}_{tile_number}_ir',
                       tiles.CLEAN_LONGWAVE_WINDOW[tile_number],
                       lats, lons,
                       cmap=get_SMN_palette(),
                       vmin=-90, vmax=50)
        close_datasets(ds)

    return f'end of {year} {day_of_year} {hour}'


async def _save_OTs_images(images_path, hours_list):
    with concurrent.futures.ProcessPoolExecutor(max_workers=len(hours_list)) as executor:
        futures = {executor.submit(_save_one_hour_OTs_images, images_path, *hour) for hour in hours_list}
        for future in concurrent.futures.as_completed(futures):
            data = future.result()
            print(data)


def get_usage():
    cpu = psutil.cpu_percent()
    # gives an object with many fields
    memory = psutil.virtual_memory()
    # you can convert that object to a dictionary
    return {'CPU%': cpu, "free GB": memory.free >> 30}


def save_OTs_images(images_path, hours_list):
    _start = time.time()
    asyncio.set_event_loop(asyncio.new_event_loop())
    loop = asyncio.get_event_loop()
    try:
        future = asyncio.ensure_future(_save_OTs_images(images_path, hours_list))
        loop.run_until_complete(future)
        print(f"Execution time: { time.time() - _start }")
    finally:
        loop.close()


# from cima.goes.gcs import mount_colab, set_credentials
# set_credentials('/Users/fido/projects/cima/Categoriza/generate_images/goes_cloud_storage/credentials.json')
# save_OTs_images('./downloads', [[2018, 53, 15]])